@@ -19,6 +19,8 @@ from .logger import Logger
 from lightgbm import train, Dataset, cv
 from .compat import pd_DataFrame, pd_Series
 from .default_lightgbm_parameters import default_parameters, make_default_tuning_space
+import warnings
+
 
 _TIMED_VARIABLE_EVENTS = [
     "mice",
@@ -169,6 +171,12 @@ class KernelDataSet(ImputedDataSet):
         random_state: Union[int, np.random.RandomState] = None,
     ):
 
+        warnings.warn(
+            "KernelDataSet is depreciated as of version 5.0.0 in favor "
+            + "of the more efficient ImputationKernel, and will be removed in a "
+            + "future release."
+        )
+
         s_init = datetime.now()
 
         super().__init__(
@@ -531,12 +539,16 @@ class KernelDataSet(ImputedDataSet):
             for var in self.imputation_order:
                 logger.log(" | " + self._get_column_name(var), end="")
                 mice_s = datetime.now()
-
+                mms = self.mean_match_subset[var]
                 x, y, cat = self._make_xy(var=var, return_cat=True)
                 self.time_log.add_variable_time(var, "make_xy", mice_s)
                 non_missing_ind = np.setdiff1d(
                     range(self.data_shape[0]), self.na_where[var]
                 )
+                if mms < len(non_missing_ind):
+                    non_missing_ind = self._random_state.choice(
+                        non_missing_ind, size=mms
+                    )
                 candidate_features = x[non_missing_ind, :]
                 candidate_target = y[non_missing_ind]
 
@@ -564,7 +576,6 @@ class KernelDataSet(ImputedDataSet):
                 imp_values = np.array(
                     self.mean_match_function(
                         mmc=self.mean_match_candidates[var],
-                        mms=self.mean_match_subset[var],
                         model=current_model,
                         candidate_features=candidate_features,
                         bachelor_features=bachelor_features,
@@ -874,6 +885,7 @@ class KernelDataSet(ImputedDataSet):
                 impute_new_data_s = datetime.now()
 
                 current_model = self.get_model(var, itergrab)
+                mms = self.mean_match_subset[var]
 
                 # Collect bachelor information
                 x, y = imputed_data_set._make_xy(var, return_cat=False)
@@ -881,12 +893,18 @@ class KernelDataSet(ImputedDataSet):
                 candidate_features, candidate_target = self._make_xy(
                     var, return_cat=False
                 )
+                non_missing_ind = np.setdiff1d(
+                    range(self.data_shape[0]), self.na_where[var]
+                )
+                if mms < len(non_missing_ind):
+                    non_missing_ind = self._random_state.choice(
+                        non_missing_ind, size=mms
+                    )
 
                 meanmatch_s = datetime.now()
                 imp_values = np.array(
                     self.mean_match_function(
                         mmc=self.mean_match_candidates[var],
-                        mms=self.mean_match_subset[var],
                         model=current_model,
                         candidate_features=candidate_features,
                         bachelor_features=bachelor_features,
